

from .Geocoding import _get_throttle_time
from .Geocoding import geocode
from .Geocoding import _query
from .Geocoding import _prepare_geocode_result
from .open_addresses import removeprefix
from .open_addresses import requestDataOpenAddresses
from .open_addresses import save
from .point_validation import convertToGeoFormat
from .point_validation import state_map
from .point_validation import getCityCode
from .point_validation import getCityLimits
from .point_validation import isContained
from .point_validation import contains
from .preprocessing import trata_abv
from .preprocessing import concat_OA_addresses
from .preprocessing import removeCaracters
from .preprocessing import splitFullAddress
from .preprocessing import prepare_data
from .String_Match import select_by_scoremin
from .String_Match import do_string_matching
from .visualization import plot_coordinates
from .visualization import visualizeOutsidePoints
from .visualization import visualizeByGeocodeService
from .visualization import get_worst_GeoAPI
from .visualization import get_mean_per_end
from .visualization import get_coord_per_end
from .visualization import coord_to_radians
from .visualization import max_dist
from .visualization import get_dispersion
from .visualization import visualizeDispersionGeo
from .visualization import visualize_dispersion
# from GeoAPI.Data import *
from .Data import load_csv

__all__ = [ "concat_OA_addresses",
            "contains",
            "convertToGeoFormat",
            "coord_to_radians",
            "do_string_matching",
            "geocode",
            "getCityCode",
            "getCityLimits",
            "get_coord_per_end",
            "get_dispersion",
            "get_mean_per_end",
            "get_worst_GeoAPI",
            "isContained",
            "load_csv",
            "max_dist",
            "plot_coordinates",
            "prepare_data",
            "removeCaracters",
            "removeprefix",
            "requestDataOpenAddresses",
            "save",
            "select_by_scoremin",
            "splitFullAddress",
            "state_map",
            "trata_abv",
            "visualizeByGeocodeService",
            "visualizeDispersionGeo",
            "visualizeOutsidePoints",
            "visualize_dispersion",
            "_get_throttle_time",
            "_prepare_geocode_result",
            "_query",
            ]


import pandas as pd
import geopandas as gpd
from sklearn.metrics.pairwise import haversine_distances
from math import radians
import numpy as np
from shapely.geometry import Point


# COLEÇÃO DE MÉTODOS UTEIS PARA OUTRAS FUNÇÕES DA BIBLIOTECA!

def coord_to_radians(list_coord:list) -> list:
    """
    This function converts coordinates to radians.


    Parameters:
    ----------
    list_coord: list
        receive a list of coordinates.

    Return:
    ------
    list:
        returns a list of tuples containing coordinates in radians.
    """

    coord_in_radians = [tuple([radians(i[0]),radians(i[1])]) for i in list_coord]
    return coord_in_radians
    
def max_dist(coord_in_radians:list) -> float:
    """
    This function receives a list of coordinates in radians and will calculate the greatest distance
    from the midpoint of the coordinates using haversine distance.

    Parameters:
    ----------
    coord_in_radians: list
        coordinate list in radians.
    Return:
    -------
    float: returns a float number representing the greatest distance from the midpoint
    """
    const = 6371000/1000 
    max_dist = round(np.nanmax(haversine_distances(coord_in_radians))*const,2)
    return max_dist
  
def convertToGeoFormat(dataframe: pd.DataFrame) -> gpd.GeoDataFrame:
    """
    This function transforms a dataframe into a geodataframe using the geometry column.

    Parameters:
    ----------
    df: DataFrame
        dataframe that will be converted

    Errors:
    ------
    RuntimeError: if the dataframe does not have the geometry column
    Return:
    -------
    dispersion_df: gpd.GeoDataFrame
        dataframe resulting from conversion
    """
    df = dataframe.copy()
    if isinstance(df.geometry,gpd.GeoSeries):
        new_df = df
    
    elif "geometry" in df.columns:
        df['geometry'] = gpd.GeoSeries.from_wkt(df['geometry'])
        new_df = gpd.GeoDataFrame(df, geometry="geometry")
    elif 'latitude' and 'longitude' in df.columns:
        df['geometry'] = gpd.GeoSeries.from_xy(df.longitude,df.latitude)
        new_df = gpd.GeoDataFrame(df, geometry="geometry")
    else:
        raise RuntimeError("The dataframe needs a column named geometry or a latitude and longitude ones")
    
    
    return new_df


import pandas as pd
from pkg_resources import resource_filename


tabela = pd.read_csv(resource_filename('GeoAPI','Data/Tabela_abreviações.csv'))
list_abv = tabela['Abreviatura'].to_list()
list_exp = tabela['Expressão'].to_list()

def trata_abv(input:list) -> list:
    '''
    Function to pre process slangs and abreviantions in the input adress.

    Parameters:
    -----------
    input:list
          list of adress to pre process.

    Returns:
    --------
    List:
        List of string contaning the processed addresses.
    '''
    out_list = []
    for item in input:
        clean = item.upper().replace('. ',' ').replace('.',' ').replace(',',', ').split()

        b = []
        for i in clean:
            if i not in list_abv:b.append(i)
            for idx, j in enumerate(list_abv):
                if i == j:
                    b.append(list_exp[idx].upper())
                    break
        
        out_list.append(' '.join(b).capitalize())
    return out_list

def concat_OA_addresses(OA_data: pd.DataFrame) -> pd.DataFrame:
    """
    
    This function will concatenate the parts of an address and make it a complete address.
    Parameters:
    ----------
    
    OA_data: dataframe with open addresses data

    Return:
    -------
    
    OA_data: Dataframe with a new column containing the full address


    
    """
    end_completo = []
    for row in OA_data.itertuples():
        number = row.number
        if number == "SN":
            number = ""
        end_completo.append("{} {} {} {}".format(row.street, number,
            row.city, row.region))

    OA_data = OA_data.assign(end_completo = end_completo)

    return OA_data

def removeCaracters(df: pd.DataFrame, column: str, caracters: list):
    """
    
    Parameters:
    ----------
    df: DataFrame
    
    column: str

    caracters: list[str]

    Return:
    -------

    df: Dataframe
    
    """
    dataframe = df.copy()
    if column in dataframe.columns:
        for c in caracters:
            dataframe[column] = dataframe[column].apply(lambda x: x.replace(c, ""))
        return dataframe
    else:
        raise ValueError("the column entered does not exist in the dataframe")

def splitFullAddress(dataframe: pd.DataFrame, address_column: str = "end_completo") -> pd.DataFrame: 
    """
    This function decomposes a string referring to the complete address into smaller parts.
    
    Parameters:
	----------
	dataframe: DataFrame
		dataframe containing a column with full address information
	
	Return:
	-------
	
	DataFrame
        A new dataframe with new columns referring to the decomposition
        of this address, with street, number, city and state.
    
    """
  
    numbers = []
    cities = []
    states = []
    addresses = []

    

    df = dataframe.copy()
    if address_column not in df.columns:
        raise ValueError("Address column does not exists in dataframe")

    for i in df.iloc():
        addresses.append(i[address_column].split(",")[0])
        numbers.append(i[address_column].split(",")[1].strip())
        cities.append(i[address_column].split(",")[2].split("-")[0].strip())
        states.append(i[address_column].split(",")[2].split("-")[1].strip())

    df['endereco'] = pd.Series(data=addresses, dtype='str')
    df['numero'] = pd.Series(data=numbers, dtype='str')
    df['cidade'] = pd.Series(data=cities, dtype='str')
    df['estado'] = pd.Series(data=states, dtype='str')

    return df

def prepare_data(df: pd.DataFrame) -> pd.DataFrame:
    """
    This function receives a dataframe, decomposes the complete address into smaller parts 
    and then this dataframe is submitted to a function that checks points outside the city's geometry.
    
    Parameters:
    ----------
    df: pd.DataFrame
        Dataframe with end_completo column

    Return:
    ------
    df: pd.DataFrame
        dataframe resulting from the transformation

    """

    df = splitFullAddress(df)
    df.cidade = df.cidade.apply(lambda x: x.lower())
    #df = batch_outside_geometry_limits(df)
    return df



import os
import sys
from pathlib import Path
import geopandas as gpd
import matplotlib.pyplot as plt
import pandas as pd
import requests
from unidecode import unidecode
import json
from shapely.geometry import Point

sys.path.append("../..")


from pkg_resources import resource_filename


def convertToGeoFormat(df: pd.DataFrame) -> gpd.GeoDataFrame:
    """
    This function transforms a dataframe into a geodataframe using the geometry column.

    Parameters:
    ----------
    df: DataFrame
        dataframe that will be converted

    Errors:
    ------
    RuntimeError: if the dataframe does not have the geometry column
    Return:
    -------
    dispersion_df: gpd.GeoDataFrame
        dataframe resulting from conversion
    """
    
    if "geometry" in df.columns:
        df['geometry'] = gpd.GeoSeries.from_wkt(df['geometry'])
        new_df = gpd.GeoDataFrame(df, geometry="geometry")
    else:
        raise RuntimeError("The dataframe needs a column named geometry")
    if "Unnamed: 0" in new_df.columns:
        new_df = new_df.drop(columns=["Unnamed: 0"], axis=1)
  
    
    return new_df

def state_map(state: str):
	"""
	This function maps the states reported as an abbreviation to the full state name.

	Parameters:
	----------
	state: str
		State abbreviation. Example: MG, SP, GO, MS
	Return:
	-------
	str: 
		Full name of state. Example: Minas Gerais, São Paulo, Goiás...
	"""

	states = {
		'AC': 'Acre',
		'AL': 'Alagoas',
		'AP': 'Amapá',
		'AM': 'Amazonas',
		'BA': 'Bahia',
		'CE': 'Ceará',
		'DF': 'Distrito Federal',
		'ES': 'Espírito Santo',
		'GO': 'Goiás',
		'MA': 'Maranhão',
		'MT': 'Mato Grosso',
		'MS': 'Mato Grosso do Sul',
		'MG': 'Minas Gerais',
		'PA': 'Pará',
		'PB': 'Paraíba',
		'PR': 'Paraná',
		'PE': 'Pernambuco',
		'PI': 'Piauí',
		'RJ': 'Rio de Janeiro',
		'RN': 'Rio Grande do Norte',
		'RS': 'Rio Grande do Sul',
		'RO': 'Rondônia',
		'RR': 'Roraima',
		'SC': 'Santa Catarina',
		'SP': 'São Paulo',

		'SE': 'Sergipe',
		'TO': 'Tocantins'
	}
	
	value = states.get(state)
	if value == None:
		raise KeyError("Do not possible find the state key")
	else:
		return unidecode(states[state].lower())

def getCityCode(state: str, city: str) -> int:
	"""
	
	This function will obtain the code of a city from an IBGE city code table. 
	This search is made through the city and its respective state.

	Parameters:
	----------
	state: str
		brazilian state.
	city: str
		city ​​of state informed.

	Return:
	------
	int:
		city code.

	"""
	codes_path = Path(__file__).parent.resolve()
	codes_path = Path.joinpath(codes_path, "relatorio.xls")

	city_codes = pd.read_excel(resource_filename('GeoAPI','Data/relatorio.xls'))

	
	
	state = unidecode(state.lower())
	city = unidecode(city.lower())
	
	if len(state) == 2:		
		state = state_map(state.upper())
		
	
	codes = city_codes.filter(items=['Nome_UF', 'Código Município Completo'
					, 'Nome_Município'])

	codes['Nome_UF'] = codes['Nome_UF'].apply(lambda x : unidecode(x.lower()))
	codes['Nome_Município'] = codes['Nome_Município'].apply(lambda x : unidecode(x.lower()))
		
	codes = codes.loc[((codes['Nome_UF'] == state) & 
							(codes['Nome_Município'] == city)), 'Código Município Completo']
	
	if codes.empty:
		return -1
	else:
		return codes.item()

def getCityLimits(city_code: int, year: int = 2020) -> json:
	
	"""
	This function receives the city code and from it, a request is made to get the city limits.

	Parameters:
	----------
	city_code: int
		code associated with the searched city.
	year: int
		period in which city limits will be obtained.
	Return:
	-------
	
	json:
		returns a json containing the city's geometry
	"""
	END_POINT = 'http://servicodados.ibge.gov.br/api/v3/malhas/municipios/'
		
	try:
		city = requests.get(END_POINT +
		 	'{}?formato=application/vnd.geo+json&qualidade=maxima&periodo={}'.format(
			 	city_code, year
		))		
		if city.status_code == 200:
			request_json = city.json()	
			return request_json
		else:
			return None
	except requests.exceptions.HTTPError as err:
		raise SystemExit(err)
	except requests.exceptions.Timeout:
		raise SystemExit()
					
def isContained(longitude: float, latitude: float, city_limits: gpd.GeoDataFrame) -> bool:
	"""
	This function checks whether a point is contained in certain geometry.
	
	
	Parameters:
	----------
	longitude: int
		longitude coordinate.
	latitude: int
		latitude coordinate.
	city_limits: QgsVectorLayer
		vector layer of city limits

	Return:
	-------
	bool:
		The function returns true if the point is contained in the geometry of the city.
		Otherwise, return false.
	"""
	point = Point(longitude, latitude)
	
	if city_limits.contains(point).bool():
		return True
	else:
		return False

def contains(state:str, city: str, longitude: float, latitude: float) -> bool:
	"""
	This function is responsible for performing the entire
	verification process to determine if a point is contained in a geometry.

	Parameters:
	----------
	state: str
		brazilian state.
	city: str
		city ​​of state informed.
	longitude: int
		longitude coordinate.
	latitude: int
		latitude coordinate.

	Return:
	-------
	
	bool:
		The function returns true if the point is contained within a city boundary.
		Otherwise, return false.
	"""
	city_code = getCityCode(state, city)
	
	if city_code != -1:
		if not isFileExists(city):
			city_limits = getCityLimits(city_code)
			save(city_limits, city)
			layer = load(city, "gpd")
			if isContained(longitude, latitude, layer):
				return True
			else:
				return False
		else:
			layer = load(city, "gpd")
			
			if isContained(longitude, latitude, layer):
				return True
			else:
				return False

	


def save(city_limits: json, city_name: str, directory_path: str = "Data/ibge/"):
	"""
	This function will be save the IBGE files, 
	which concern the limits of a certain city. '

	Parameters:
	-----------
	city_limits: json
		response to the request to IBGE
	city_name: str
		name of geojson file.
	directory_path: str
		path to save json. 
	"""
	try:
		os.makedirs(directory_path, exist_ok = True)
	except OSError as error:
		raise SystemError()
	
	with open(directory_path+ '{}_ibge.geojson'.format(city_name.lower()), 'w') as f:
			json.dump(city_limits, f)



def load(city_name: str, return_type: str, directory_path: str = 'Data/ibge/'):
	"""
	This function will load a geojson referring to the boundaries of a municipality
	and from a return type it returns an object. 

	Parameters:
	----------
	city_name: str 
		symbolizes the name of the file to be searched.
	return_type: str  
		the return type will tell if the method will return a qgis vector layer 
				or a geo dataframe.
	directory_path: str
		path to load a json.

	Returns:
	--------
	QgsVectorLayer: 
		returns the geojson of the city entered in vector layer format.
	GeoDataFrame:
		returns the geojson of the city entered in GeoDataFrame format. 
	"""
	
	if unidecode(return_type.lower()) == "qgis":
		pass
		#return QgsVectorLayer(directory_path+'{}_ibge.geojson'.format(city_name),
		 #			'{}'.format(city_name), 'ogr')

	elif unidecode(return_type.lower()) == "gpd":
		return gpd.read_file(directory_path
							+'{}_ibge.geojson'.format(city_name.lower()))

def isFileExists(city_name:str, directory_path: str = "Data/ibge/") -> bool:
	"""
	This function checks if a file already exists in a specific directory.

	Parameters:
	-----------
	city_name: str
		Files are saved by city name.
	directory_path: str
		Directory where these files were saved.
	
	Return:
	------
	bool: 
		True -> if file exists
		False -> otherwise
	"""
	if os.path.exists(directory_path
						+"{}_ibge.geojson".format(city_name.lower())):
		return True
	else:
		return False
		
def renderCityLimits(city:str):
	"""
	This function will display a graph representing the boundaries of a particular city.

	Parameters:
	----------
	city: string
		the city we want to show.

	"""
	city_limits = load(city, "gpd")	
	city_limits.plot()
	plt.show()	

def outside_geometry_limits(points : list, state: str, city = []) -> pd.DataFrame:
	"""
	This function will analyze if a certain point is contained
	in one of the cities present in the list of cities.


	Parameters:
	-----------
	latitude: float
		latitude coordinate
	longitude: float
		longitude coordinate
	state: str
		Brazilian state
	city: list
		list of cities to check

	Return:
	-------
	DataFrame:
		This dataframe will contain the answer if a certain point is contained for a list of cities.
	
	"""
	index = city
	
	dataframe = pd.DataFrame(columns=['latitude', 'longitude', 'estado', 'outside'])	
	
	if len(city) < 1:
		raise Exception("it is necessary to inform at least one city") 
	if len(city) == 1:
		city = city[0]
	

	lines = []
	for city in index:
		for point in points:
			if contains(state=state, city=city, longitude=point[0],latitude=point[1]):				
				line = pd.DataFrame([{"latitude" : point[1], "longitude": point[0],
					"estado": state, "outside": 0,}], index=[city])
				
				lines.append(line)
			else:
				line = pd.DataFrame([{"latitude" : point[1], "longitude": point[0],
					"estado": state, "outside": 1,}], index=[city])
				
				lines.append(line)
				
	for i in lines:
		dataframe = dataframe.append(i, ignore_index=False)


	dataframe = dataframe.reset_index().rename(columns={"index": "cidade"})

	dataframe = dataframe.groupby(by=["cidade", "estado" ]).apply(lambda x: x.sort_index())

	dataframe.index = dataframe.index.droplevel(-1)
	
	dataframe.drop(columns=["cidade", "estado"], axis=1, inplace=True)
	
	
	
	return dataframe
	
def batch_outside_geometry_limits(geocoded_df: pd.DataFrame):		
	"""
	This function checks if a set of points is contained within the limits of a city, 
	creating a column that is responsible for symbolizing which ones are inside or outside the limits.
	
	Parameters:
	-----------
	
	geocoded_df: DataFrame 
		a set of geocoded points that will be tested
	
	Return:
	------
	GeoDataFrame:
		The return is a GeoDataFrame with the marking of points that are outside the city limits.
	"""

	outside = []

	df = geocoded_df.copy()
    
	if type(df) != gpd.GeoDataFrame:
		df = convertToGeoFormat(df)
	
	if "geometry" in df.columns:
		for row in df.itertuples():
			if contains(row.estado, row.cidade, row.geometry.x, row.geometry.y):
				outside.append(0)
			else:
				outside.append(1)
	elif (("latitude" in df.columns) & ("longitude" in df.columns)):
		for row in df.itertuples():
			if contains(row.estado, row.cidade, longitude = row.longitude, latitude=row.latitude):
				outside.append(0)
			else:
				outside.append(1)

			
	df = df.assign(outside = outside)
	#geocoded_df.insert(0, "outside", outside, True)
	return df







from utils import convertToGeoFormat, coord_to_radians, max_dist
import pandas as pd
from sklearn.metrics.pairwise import haversine_distances
import numpy as np


def get_coord_per_end(dispersion_df: pd.DataFrame, multi_sum) -> list:
    """

    Parameters:
    -----------
    dispersion_df: DataFrame
        This dataframe containing the adreesses
    multi_sum: DataFrame
        This dataframe containing latitudes and longitudes of each API for each address in new_df
    Return:
    ------
    coord_per_end: list
        Return a list with a coordinate of each addres generated by each API
    """
    coord_per_end = []
    for end in dispersion_df.end_completo:
        x = [tuple(x) for x in multi_sum.xs(end).values]
        coord_per_end.append(x)
    return coord_per_end


def get_mean_per_end(multi_sum) -> list:
    """
    This function will group the values ​​by address and calculate the average 
    of the latitudes and longitudes.

    Parameters:
    ----------
    multi_sum: pd.DataFrame
        This dataframe containing latitudes and longitudes of each API for each address in dispersion_df
    Return:
    ------
    Two lists:
        mean of longitude and mean of latitude
    """
    lat_mean = []
    lon_mean = []
    for end in multi_sum.index.get_level_values('end_completo').unique():
        x = multi_sum.xs(end).longitude.mean()
        lon_mean.append(x)
        y = multi_sum.xs(end).latitude.mean()
        lat_mean.append(y)
    return lon_mean, lat_mean


def get_worst_GeoAPI(multi_sum):
    """
    This function is used in get_dispersion. It will get the API that has 
    the worst dispersion.

    Parameters:
    ----------
    dispersion_df: pd.DataFrame
        This dataframe containing the adreesses
    multi_sum: pd.DataFrame
        This dataframe containing latitudes and longitudes of each API for each address in dispersion_df

    Return:
    ------

    worst_api : list
        Return a list containing the worst api for each address
    """
    worst_api = []
    for end in multi_sum.index.get_level_values('end_completo').unique():
        y = tuple(multi_sum.xs(end).mean().values)
        y = coord_to_radians([y])
        x = [tuple(x) for x in multi_sum.xs(end).values]
        x   = coord_to_radians(x)
        worst = multi_sum.index.get_level_values('GeoAPI').unique()[
            np.nanargmax(haversine_distances(x, y))]
        worst_api.append(worst)
    return worst_api


def get_dispersion(geocoded_data, metrics: list) -> pd.DataFrame:
    """
    This function will do the scatter calculation based on a metric and 
    will return a new dataframe containing this information.

    Parameters:
    ----------
    geocoded_data: DataFrame,dict or str format
        Data containing the informations to get dispersion
        must contais the following coluns:
            latitude -
            longitude -
            geoAPI -
            end_completo -

    metric: str
        dispersion metric that will be used

    Return:
    ------
    DataFrame:
        This function will be return a new dataframe with a new column "dispersion"
    """
    geo_df = geocoded_data.copy(deep=True)

    if isinstance(geo_df, dict):
        geo_df = pd.DataFrame(geo_df)
        geo_df = convertToGeoFormat(geo_df)

    elif isinstance(geo_df, pd.DataFrame):
        geo_df = convertToGeoFormat(geo_df)

    # try:
    #     disp_metric = metrics.pop()
    #     print(disp_metric)

    # except IndexError:
    #     raise IndexError("it is necessary to inform at least one metric")
    # except AttributeError:
    #     raise AttributeError('Input must be a list')

    geo_df['longitude'] = geo_df.geometry.\
    apply(lambda p: p.x if not p.is_empty else None)

    geo_df['latitude'] = geo_df.geometry.\
    apply(lambda p: p.y if not p.is_empty else None)

    multi_sum = geo_df.set_index(['end_completo', 'GeoAPI'])[
        ['latitude', 'longitude']]
    dispersion_df = pd.DataFrame()
    dispersion_df = dispersion_df.assign(
        end_completo=multi_sum.index.get_level_values('end_completo').unique())


    lon, lat = get_mean_per_end( multi_sum)
    dispersion_df.insert(0, "longitude", lon, True)
    dispersion_df.insert(0, "latitude", lat, True)


    worst_api = get_worst_GeoAPI(multi_sum)
    dispersion_df.insert(0, "worst_api", worst_api, True)


    max_list = []
    for disp_metric in metrics:
        if disp_metric == "DistanceFromMean":
            for coord in get_coord_per_end(dispersion_df, multi_sum):
                _max = max_dist(coord_to_radians(coord))
                max_list.append(_max)
        dispersion_df.insert(0, 'Raio', max_list, True)



    return dispersion_df



from visualization import visualize_dispersion
import pandas as pd

df = pd.read_csv('GeoAPI/Data/out.csv')
visualize_dispersion(df)
#example
from GeoAPI.visualization import *
from GeoAPI.point_validation import batch_outside_geometry_limits
from GeoAPI.preprocessing import *
from pkg_resources import resource_filename

df = pd.read_csv(resource_filename('GeoAPI','Data/data2.csv'))
gs = pd.read_csv(resource_filename('GeoAPI', 'Data/id29-clientes-mapeado_consolidado.csv'), sep=";",
                encoding="latin-1")
print(df.columns)

plot_coordinates(df, hover_data=None)

visualizeByGeocodeService(df, hover_data=None, 
            address_column="end_completo",hover_name=None)

df2 = splitFullAddress(df)

new_df = batch_outside_geometry_limits(df2)

visualizeOutsidePoints(new_df, column_color="outside", hover_name=None, hover_data=None)

disp_df = get_dispersion(new_df, metrics=["DistanceFromMean"])

visualize_dispersion(disp_df, hover_name=None, hover_data=None, 
                    metric="DistanceFromMean")




import pandas as pd
import plotly.graph_objects as go
import plotly.express as xp
import geopandas as gpd

import dash
import dash_core_components as dcc
import dash_html_components as html
from dash.dependencies import Input, Output

from GeoAPI.point_validation import *

def plot_coordinates(df: pd.DataFrame, hover_data: list = None, hover_name: str = None,):
    """
    This function will display a coordinate map of the geocoding process
    
    Parameters:
    -----------
    dataframe: pd.DataFrame:
        dataframe with coordinates of geocoding. 
    
    Return:
    ------
    Nothing

    """
    dataframe = df.copy()
    if type(dataframe) != gpd.GeoDataFrame:
        dataframe = convertToGeoFormat(dataframe)
    
    if "geometry" in dataframe.columns:
        dataframe['latitude'] = dataframe.geometry.\
    apply(lambda p: p.y if not p.is_empty else 1)
        dataframe['longitude'] = dataframe.geometry.\
    apply(lambda p: p.x if not p.is_empty else 1)


    if hover_name == None:
        hover_name = df.index

    if hover_data == None:
        try:
            hover_data = df.columns.to_list().remove('geometry')
        except:
            pass


    fig = xp.scatter_mapbox(dataframe, lat = 'latitude' ,lon = 'longitude',
                         hover_data = hover_data, hover_name = hover_name ,
                        color_discrete_sequence = ["salmon"], zoom = 10, height = 1000)
    fig.update_layout(mapbox_style = "open-street-map")
    fig.update_layout(margin = {"r":0,"t":0,"l":0,"b":0})
    fig.show()

def visualizeOutsidePoints(df: pd.DataFrame, column_color: str = "outside", 
                            hover_name: str = None, hover_data: list = None):
    """
	This function receives a dataframe that will be used 
    to render the points that are or are not contained in the city's geometry.

	Parameters:
	----------
	df: DataFrame
		dataframe containing information whether the points are contained or not in the city's geometries
	
	Return:
	-------
	
	Nothing
	
    """
    dataframe = df.copy()
    if type(dataframe) != gpd.GeoDataFrame:
        dataframe = convertToGeoFormat(dataframe)
    
    if "geometry" in dataframe.columns:
        dataframe['latitude'] = dataframe.geometry.y
        dataframe['longitude'] = dataframe.geometry.x
    

    if "Unnamed: 0" in df.columns:
        df = df.drop(columns="Unnamed: 0", axis=1)
    
    if hover_name == None:
        hover_name = df.index

    if hover_data == None:
        hover_data = df.columns.to_list()

    
    
    

    dataframe['outside'] = dataframe['outside'].apply(lambda x : "yes" if x == 1 else "no")
    fig = xp.scatter_mapbox(dataframe, lat = "latitude", lon = "longitude",
                        hover_name = hover_name, hover_data = hover_data,
                         zoom = 4, height = 1000, color=column_color,
                         size_max=8, color_discrete_map={
                             "yes": "red",
                             "no": "blue",
                              
                         }, category_orders={"outside": ["yes", "no"]})
 
    fig.update_layout(mapbox_style = "open-street-map", coloraxis_showscale=False,
                    showlegend=True,
                    legend=dict(
                        traceorder="normal",
                        yanchor="top",
                        y=0.97,
                        font=dict(size=20)
                    ),
                    margin = {"r":0,"t":0,"l":0,"b":0},
                    
    )

    fig.show()
    
def visualizeByGeocodeService(df: pd.DataFrame, address_column: str, 
        hover_name: str = None, hover_data: list = None):
    """
    This function receives a dataframe that contains several points referring 
    to different geocoding API's and will show these points differentiated by API.

    Parameters:
    ----------
    df: pd.DataFrame
        This dataframe needs a GeoAPI column to differentiate the points
    
    address_column: str

    hover_name: str

    hover_data: list 
    Return:
    ------

    Nothing
    """
    fig = go.Figure()

    dataframe = df.copy()
    if type(dataframe) != gpd.GeoDataFrame:
      dataframe = convertToGeoFormat(dataframe)
    
    dataframe.geometry = dataframe.geometry.apply(lambda x: Point(0,0) if x.is_empty else x)
    dataframe.address.fillna('ADDRESS_ERROR',inplace=True)
    
    color_map = ('#0000FF','#FF0000','#FFA500','#8B008B','#FF6F9C','#343a40','#008000')
    opaci_map = (0.80,0.80,0.85,0.80,0.85,0.75,0.70)


    if address_column not in df.columns:
         raise ValueError("Address columns does not exists in dataframe")

    i = 0
    fig = go.Figure()
    for api in dataframe.GeoAPI.unique():
        fig.add_trace(go.Scattermapbox(lat = dataframe.loc[dataframe['GeoAPI'] == api].geometry.y, 
                      lon = dataframe.loc[dataframe['GeoAPI'] == api].geometry.x, mode='markers+text', 
                      marker=dict(size=10, color = color_map[i], opacity = opaci_map[i]), 
                                       name=api, text=(dataframe.loc[dataframe['GeoAPI'] == api,address_column],dataframe.loc[dataframe['GeoAPI'] == api,'end_completo'])
                                       ))
        i += 1

    fig.update_layout(
        mapbox = {
            'style':"open-street-map",
            'zoom': 4.2,
            'center': dict(
                lon=-51.92528,
                lat=-14.235004,
            )
        },
        legend = dict(
            font=dict(
                size = 20,
            ),
            y = 0.96,
            yanchor="top",
        )    
    )
    fig.update_layout(margin = {"r":0,"t":0,"l":0,"b":0})
    fig.show()

def visualizeDispersionGeo(dispersion_df, metric: str,
                            hover_name: str = None, hover_data: list = None,):
    """
    Renders a map containing geographic points and information associated with them, 
    such as: full address, the result of a dispersion based on a metric and 
    which geoapi obtained the worst result in relation to the dispersion value.
    
    Parameters:
    ----------
    dispersion_df

    Return:
    -------
    This function will render a map on https://127.0.0.1
    """
    if "Unnamed: 0" in dispersion_df.columns:
        dispersion_df = dispersion_df.drop(columns=["Unnamed: 0"], axis=1)

    if hover_name == None:
        hover_name = dispersion_df.index

    if hover_data == None:
        hover_data = dispersion_df.columns.to_list()

    fig = xp.scatter_mapbox(dispersion_df, lat = "latitude", lon = "longitude",
                        hover_data = hover_data, hover_name = hover_name,
                        zoom = 4, height = 1000, color=metric,
                        size_max=15,  color_continuous_scale="rainbow")
 
    fig.update_layout(mapbox_style = "open-street-map", showlegend=True,
                  margin = {"r":0,"t":0,"l":0,"b":0},
                  legend=dict(yanchor="top",y=0.97,))

    fig.show()

def visualize_dispersion(dispersion_df, hover_name: str = None, 
                        hover_data: list = None ,metric : str ="DistanceFromMean"):
    """
    It will plot a map with a point for each address, this point will have the worst dispersion and the API that 
    obtained the worst dispersion.
    
    Parameters:
    ----------
    new_df: DataFrame

    Return:
    ------
    Nothing
    """

    min_v = dispersion_df[metric].min()
    max_v = dispersion_df[metric].max()
    if "Unnamed: 0" in dispersion_df.columns:
        dispersion_df = dispersion_df.drop(columns=["Unnamed: 0"], axis=1)
    
    if hover_name == None:
        hover_name = dispersion_df.index
    
    if hover_data == None:
        hover_data = dispersion_df.columns.to_list()
     
    app = dash.Dash()
    app.layout = html.Div([
        dcc.Graph(id="scatter-plot"),
        html.P("Distancia entre APIs:"),
        dcc.RangeSlider(
            id='range-slider',
            min=min_v, max=max_v, step=0.1,
            marks={min_v: str(min_v), max_v: str(max_v)},
            value=[min_v, max_v]),
        
    ])

    @app.callback(
        Output("scatter-plot", "figure"), 
        [Input("range-slider", "value")])
    
    def update_bar_chart(slider_range):
        low, high = slider_range
        filter1 = dispersion_df[metric] <= high
        filter2 = dispersion_df[metric] >= low
        clean_df = dispersion_df.where(filter1&filter2)

        fig = xp.scatter_mapbox(clean_df, lat = "latitude", lon = "longitude",
                                hover_name = hover_name, hover_data = hover_data,
                                zoom = 4, height = 600, color="{}".format(metric),
                                size_max=10,color_continuous_scale='rainbow')

        fig.update_traces(marker=dict(size=8),
                        selector=dict(mode='markers'))

        fig.update_layout(mapbox_style = "open-street-map",
                            showlegend=True,
                            legend=dict(
                                yanchor="top",
                                y=0.97,
                            ),
                            margin = {"r":0,"t":0,"l":0,"b":0})
        
    
        return fig

    app.run_server(debug=True, use_reloader=False) 




from setuptools import setup, find_packages

INSTALL_REQUIRES = [
    'geopandas',
    'unidecode',
    'sparse_dot_topn',
    'dash',
    'pandas', 
    'plotly',
    'geopy==2.2.0', 
    'sklearn',
    'numpy',
    'scipy',
    'matplotlib',
    'requests' 


]


setup(
    name='terralabgeoapi',
    packages=find_packages(exclude=["*test*"]),
    version='0.0.6',
    description='GeoAPI do TerraLab',
    author='TerraLab',
    author_email='alan.ufop@gmail.com',
    url = 'https://gitlab.com/ufopterralab/data-analytics-group/geoapi.git',
    install_requires= INSTALL_REQUIRES,
    setup_requires=['pytest-runner'],
    tests_require=['pytest==4.4.1'],
    test_suite='tests',
    package_data = {"GeoAPI": ['Data/*.csv', 'Data/*.xls'] },
)


import pandas as pd
import numpy as np
import csv
from importlib import resources
 
def load_csv (data_name, as_frame = False):
    """
        Load 'data_name' from the local library
        
        Parameters 
        ----------
        data_name : str
            Name of csv file to be loaded :
            'data_pelias.csv'
            'data.csv'
            'data2.csv'
            'id29-clientes-mapeado_consolidado.csv'
            'out.csv'
            'Tabela_abreviações.csv'

        as_frame : bool
            Load the data as a DataFrame.
            Default : dict

        Returns: 
        --------
        data : dict or DataFrame
            A container with the data
    """
    data_path = "GeoApi/Data/" + data_name
    file_ = open(data_path, 'r')
    data = csv.reader(file_)
    columns = next(data)
    values = []
    for row in data :
        values.append(row)
    data_frame = pd.DataFrame(values, columns=columns)

    if as_frame :
        return data_frame
    else:
        dict_ = data_frame.to_dict()
        return dict_



import pandas as pd 
import csv

def same_char(word1:str, word2:str)->int:
    """ 
        This function calculates how many characters in word1 are 
        equals in word2 and returns your equivalence. The occurrence 
        of the characters is important. 

        Parameters
        ----------
        word1 : str
        Word that will be compared 

        word2 : str
        Base word, that will be the target of comparade

        Returns
        -------
        percent : double
        correlation between word1 and word2

        Examples
        --------
        >>>same_char('fy','fly')
        0.6666666666666666
        >>>same_char('fly','fly')
        1.0
    """
    word1_size = len(word1)
    word2_size = len(word2)
    initial_word2 = 0
    equals = 0
    for i in range(0,word1_size):
        local = word2[initial_word2:word2_size].find(word1[i])
        if  local != -1:
            initial_word2 += local + 1
            equals += 1
    percent = equals/word2_size
    return percent

def complete_word(word:str):
    """
        This function is used to standardise 'geographical words'
        (in portugues)
        
        Parameters
        ----------
        word: str
        The word that will be transformed

        Results
        -------
        word_return : str
        Word standardized

        Examples
        --------
        >>>word = complete_word('av')
        >>>print(word)
        AVENIDA
        >>>word = complete_word('aneida')
        >>>print(word)
        AVENIDA

    """
    data_path = "GeoAPI/Data/Tabela_abreviações.csv"
    file_ = open(data_path, 'r')
    data = csv.reader(file_)
    columns = next(data)
    values = []
    for row in data :
        len_row = len(row)
        for i in range(0,len_row):
            row[i] = row[i].upper()
        values.append(row)
    word = word.upper()
    count_equals = 0
    word_return = None
    for row in values:
        if((word == row[2]) or (word == row[1])):
            word_return = row[1]
    if word_return == None:
        for row in values:
            count_aux = same_char(word, row[1])
            if count_aux > count_equals:
                count_equals = count_aux
                word_return = row[1]
    return word_return



from GeoAPI import Geocoding
import geopandas as gpd
import pandas as pd

providers = {

             'google': {
                    'config': {
                           'domain':'maps.google.com.br',
                           'api_key':'AIzaSyAnJfHQRB9x4DLZyC5_6WIvawPTwVVbFqg'},
                    'region': 'br'}}



data = pd.read_csv('/home/alanss/ml/Data Analytics - TerraLAB/GeoAPI - library/GeoAPI/Data/id29-clientes-mapeado_consolidado.csv', sep=';',
                   encoding='latin-1', usecols=['end_completo', 'longitude', 'latitude'])

geo_data = gpd.GeoDataFrame(data, geometry=gpd.points_from_xy(
    data.longitude, data.latitude)).sample(n=5, random_state=1)

end = geo_data['end_completo']
cord = geo_data.geometry

dataframe = gpd.GeoDataFrame()
for provider, configs in providers.items():
    df,_ = Geocoding.geocode(end, provider=provider, **configs)
    dataframe = pd.concat([dataframe,df],axis=0,)

dataframe.to_csv('data.csv')
